@@ -515,11 +515,11 @@ def recruitInstances( nWorkersWanted, launchedJsonFilePath, launchWanted, result
         with open( os.path.expanduser('~/.ssh/known_hosts'), 'a' ) as khFile:
             jsonToKnownHosts.jsonToKnownHosts( startedInstances, khFile )
         
+        goodInstances = []
         if not g_.deviceLocsWanted:
             goodInstances = startedInstances
-        else:
+        elif startedInstances:
             returnCodes = pushDeviceLocs( startedInstances )
-            goodInstances = []
             badInstances = []
             for index, rc in enumerate( returnCodes ):
                 if rc == 0:
@@ -533,11 +533,21 @@ def recruitInstances( nWorkersWanted, launchedJsonFilePath, launchWanted, result
                 terminateInstances( args.authToken, badIids )
                 logger.info( 'purging host keys')
                 purgeHostKeys( badInstances )
-
+        if not goodInstances:
+            return []
         # install something on goodInstances, if required, else just return goodInstances
         if (not getInstallerCmd()) and (not args.commonInFilePath):
             return goodInstances
         if not sigtermSignaled():
+            def waiter():
+                thr = threading.current_thread()
+                while not thr.stopRequested:
+                    time.sleep( 30 )
+                    if not thr.stopRequested:
+                        logger.info( 'waiting for installer' )
+            narrator = threading.Thread( target=waiter )
+            narrator.stopRequested = False
+            narrator.start()
             installerCmd = getInstallerCmd()
             logger.info( 'calling tellInstances to install on %d instances', len(goodInstances))
             stepStatuses = tellInstances.tellInstances( goodInstances, installerCmd,
@@ -546,6 +556,11 @@ def recruitInstances( nWorkersWanted, launchedJsonFilePath, launchWanted, result
                 timeLimit=min(args.instTimeLimit, args.timeLimit), upload=args.commonInFilePath, stopOnSigterm=not True,
                 knownHostsOnly=True
                 )
+            narrator.stopRequested = True
+            narrator.join( timeout=60 )
+            if narrator.is_alive():
+                logger.warning( 'the narrator thread did not stop')
+
             # SHOULD restore our handler because tellInstances may have overridden it
             #signal.signal( signal.SIGTERM, sigtermHandler )
             if not stepStatuses:
@@ -1170,6 +1185,9 @@ def runBatch( **kwargs ):
         settingsToSave = argsToSave.copy()
         with open( settingsJsonFilePath, 'w' ) as settingsFile:
             json.dump( settingsToSave, settingsFile )
+        # return early if recruitOnly
+        if args.recruitOnly:
+            return int( len( goodInstances ) == 0 )  # zero if good, 1 if bad
 
         if not len(goodInstances):
             logger.error( 'no good instances were recruited')
@@ -1259,6 +1277,8 @@ def createArgumentParser():
         default=0 )
     ap.add_argument( '--limitOneFramePerWorker', type=boolArg, help='prevent any worker from doing multiple frames',
         default=False )
+    ap.add_argument( '--recruitOnly', type=boolArg, help='set True to exit immediately after installing',
+        default=False )
     ap.add_argument( '--autoscaleInit', type=float, help='multiple (instances per frame) to launch initially',
         default=1 )
     ap.add_argument( '--autoscaleMax', type=float, help='maximum multiple (instances per frame) to keep active',
